import click
import sys
import traceback
import time
import json
from . import tabulate
from . import websocket as ws
from . import commentjson as commentjson

__all__ =['Critical','Error','Warning','Info','echo','cli','error','warning','debug','info','log','log_json','log_table','critical','fatal','sleep','set_output_filter','ws','commentjson']


## GLOBAL OPTIONS
_options = {
    "colors": True,
    "traceback": True
}


## Styles
_styles = {
    "Critical":{"fg":"magenta","bold":True},
    "Error":{"fg":"red","bold":True},
    "Warning":{"fg":"yellow"},
    "Info":{"fg":"green"}
}



class Style():
    def __init__(self,val):
        self.val = val

    def __str__(self):
        if _options["colors"]:
            return click.style(str(self.val),**_styles[self.__class__.__name__])
        else:
            return str(self.val)

class Critical(Style):
    pass

class Error(Style):
    pass

class Warning(Style):
    pass

class Info(Style):
    pass

## The magical Echo

def echo(*args,**kwargs):
    sep = kwargs.get("sep"," ")
    end = kwargs.get("end","\n")
    kwargs.pop("sep",None)
    kwargs.pop("end",None)
    if args:
        click.echo(str(args[0]),nl=False,**kwargs)
        for i in range(1,len(args)):
            click.echo(str(sep),nl=False,**kwargs)
            click.echo(str(args[i]),nl=False,**kwargs)
    click.echo(str(end),nl=False,**kwargs)


def critical(*args,**kwargs):
    if "exc" in kwargs:
        exc = kwargs.pop("exc")
    else:
        exc = None
    if exc:
        if _options["traceback"]:
            echo(Critical("[fatal]>"),*args,err=True,**kwargs)
            traceback.print_exc()
        else:
            echo(Critical("[fatal]>"),str(exc),*args,err=True,**kwargs)
    else:
        echo(Critical("[fatal]>"),*args,err=True,**kwargs)
    sys.exit(2)
    

def fatal(*args,**kwargs):
    echo(Error("[error]>"),*args,err=True,**kwargs)
    sys.exit(1)

def error(*args,**kwargs):
    echo(Error("[error]>"),*args,err=True,**kwargs)

def warning(*args,**kwargs):
    echo(Warning("[warning]>"),*args,err=True,**kwargs)

def info(*args,**kwargs):
    if not output_filter: return
    echo(Info("[info]>"),*args,**kwargs)

def log(*args,**kwargs):
    if not output_filter: return
    echo(*args,**kwargs)

def debug(*args,**kwargs):
    if not output_filter or not _options["verbose"]: return
    echo(Info("[debug]>"),*args,**kwargs)


def log_json(js,*args,**kwargs):
    if not output_filter: return
    cls = kwargs.pop("cls",None)
    sort_keys = kwargs.pop("sort_keys",False)
    echo(json.dumps(js,indent=indent,cls=cls,sort_keys=sort_keys),*args,**kwargs)

def log_table(table,*args,**kwargs):
    if not output_filter: return
    headers = kwargs.pop("headers",[])
    echo(tabulate.tabulate(table,headers),*args,**kwargs)

def set_output_filter(enabled):
    global output_filter 
    output_filter = enabled

output_filter = True
indent = None

def sleep(n):
    time.sleep(n)
## Main entrypoint gathering 

@click.group(help="Zerynth Installer.")
@click.option("-v","verbose",flag_value=True,default=False,help="Verbose.")
@click.option("--colors/--no-colors","nocolors",default=True,help="To enable/disable colors.")
@click.option("--traceback/--no-traceback","notraceback",default=False,help="To enable/disable exception traceback printing on criticals.")
@click.option("--user_agent",default="ztc",help="To insert custom user agent.")
@click.option("--pretty","pretty",flag_value=True, default=False,help="To display pretty json output.")
@click.option("-J","__j",flag_value=True,default=False,help="To display the output in json format.")
def cli(verbose,nocolors,notraceback,user_agent,__j,pretty):
    _options["colors"]=nocolors
    _options["traceback"]=notraceback
    _options["verbose"]=verbose
    global ua
    ua = user_agent
    global human
    human = not __j
    global indent 
    indent = 4 if pretty else None
