var os = require("os")
var fs = require("fs-extra")
var path = require("path")
var http = require("http")
var https = require("https")
var win = nw.Window.get()
var mkdirp = require('mkdirp');
var cp = require('child_process')
var rimraf = require("rimraf")


//hijack console

var __log = [];
var oldf = console.log;
console.log = function(){
    __log.push(JSON.stringify(arguments))
    oldf.apply(console,arguments)
}

function get_env(){
    var env = {}
    env.home = os.homedir()
    if (process.env.ZERYNTH_TESTMODE == 2) {
        if (os.platform().startsWith("win")) env.zdir = path.join(env.home,"zerynth2_test");
        else env.zdir = path.join(env.home,".zerynth2_test");
    } else {
        if (os.platform().startsWith("win")) env.zdir = path.join(env.home,"zerynth2");
        else env.zdir = path.join(env.home,".zerynth2");
    }

    env.platform = (os.platform().startsWith("win")) ? ("windows64"):(   (os.platform().startsWith("linux")) ? ("linux64"):("mac"))
    env.cfg       = path.join(env.zdir,"cfg")
    env.tmp       = path.join(env.zdir,"tmp")
    env.sys       = path.join(env.zdir,"sys")
    env.dist      = path.join(env.zdir,"dist")
    env.cwd       = process.cwd()
    env.installer_version="r2.0.11"  //the online version is checked against this one
    env.reinstalling = fs.existsSync(env.cfg) ? "ok":"new"
    if (process.env.ZERYNTH_TESTMODE == 1) {
        env.remote    = "http://localhost"
        console.log("LOCAL")
    } else if (process.env.ZERYNTH_TESTMODE == 2) {
        env.remote    = "https://test.zerynth.com"
        console.log("CI")
    } else {
        env.remote    = "https://backend.zerynth.com"
        console.log("REMOTE")
    }
    env.remote_pack_list = function(vrs){    
        return env.remote+"/repository/"+vrs+"?p="+Math.floor(Date.now())
    }
    env.remote_versions_list = function(){    
        return env.remote+"/versions?p="+Math.floor(Date.now())
    }
    env.remote_installer_version = function(){    
        return env.remote+"/installer/installer.json?p="+Math.floor(Date.now())
    }
    mkdirp.sync(env.tmp)
    mkdirp.sync(env.cfg)
    mkdirp.sync(env.dist)
    var pycmd;
    var pydir = path.join(env.cwd,"python")
    if (env.platform.startsWith("win")) {
        pycmd = path.join(pydir,"python.exe")
    } else if(env.platform.startsWith("mac")){
        //mac needs python3 call!
        pycmd = path.join(pydir,"bin","python3")
    } else {
        pycmd = path.join(pydir,"bin","python")
    }
    ZTC.set_python(pycmd)
    ZTC.set_toolchain(path.join(env.cwd,"install.py"))
    ZTC.set_locale(path.join(env.cwd,"ztc","loc.py"))
    return env
}


var env = get_env();


function fix_runtime(elem){
    clog("Fixing runtime ...")
    var newelem = path.join(env.sys,"new"+elem)
    var elem = path.join(env.sys,elem)
    return new Promise((resolve,reject)=>{
        if (fs.existsSync(newelem)) {
            //update runtime
            clog("Fixing runtime: "+newelem+" to "+elem)
            // rimraf.sync(python)
            // fs.renameSync(newpython,python)
            
                var args = ["rename",newelem,elem]
                ZTC.command(args,{
                    stdout: (line)=>{
                        log(line) //log everything
                    }
                }).then(()=>{
                    console.log(elem+" renamed")
                    resolve()
                }).catch((err)=>{
                    reject("fix_runtime")
                })
        }
        else{
            resolve()
        }
    })
}

function track_installation(checkpoint,tag){
    var url = env.remote+"/v1/track_installation/"+checkpoint+"/"+tag+"/"+env.platform
    $.ajax({
        url: url
    }).then((data)=>{
        console.log("Tracking "+checkpoint+" ok")
    },(err)=>{
        console.log("Tracking "+checkpoint+" ko: "+err)
    })
}




function clog(msg){
    //__log.push(msg)
    console.log(msg)
}
function log(msg){
    //__log.push(msg)
    console.log(msg)
    var res = msg.match(/^\[(.*)]>(.*)/)
    if (res) {
        if (res[1]=="info") $(".dtext").html(res[2])
    } else {
        $(".dtext").html(msg)
    }
}



var previous_release = null;
function startup(){
    var mjfile = path.join(env.tmp,"major_release.json")
    if(
        !fs.existsSync(path.join(env.cfg,"installation.json")) ||
        fs.existsSync(mjfile)
        ){
        if (fs.existsSync(mjfile)){
            try {
                clog("major_release!")
                var js = fs.readFileSync(mjfile,{encoding:"utf-8"})
                fs.removeSync(mjfile)
                var jsp = JSON.parse(js)
                previous_release = jsp.version
            } catch(err){
                clog(err)
            }
        } else {
            //destroy old cfg/env (maybe it's an old zerynth installation)
            rimraf.sync(env.cfg)
            rimraf.sync(path.join(env.zdir,"env"))
            make_new_dir(env.cfg)
        }
        //check for xz
        return "install"
    }
    return "studio"
}

function get_title(){
    if (process.env.ZERYNTH_TESTMODE == 1) {
        return "LOCAL MODE"
    } else if (process.env.ZERYNTH_TESTMODE == 2) {
        return "CI MODE"
    }
    return "Zerynth"
}

function show_log(){
    fs.writeFileSync(path.join(env.cfg,"install.log"),__log.join("\n"))
    nw.Window.open("file://"+path.join(env.cfg,"install.log"),{title:"Zerynth Installer Log",new_instance:false})
}

function send_log(){
    var mylog = {
        timestamp: new Date(),
        platform: os.platform(),
        release: os.release(),
        locale: ZTC.locale,
        instpath: env.dist,
        srcpath: env.cwd,
        lines: __log
    }
    var url = env.remote+"/v1/track_installation"
    $.ajax({
        url: url,
        method:"POST",
        data:JSON.stringify(mylog),
    }).then((data)=>{
        console.log("Tracking log ok")
        $(".zmessage").html('Thanks! Check our <a href="javascript:nw.Shell.openExternal(\'https://community.zerynth.com\')">forum</a> for help!')
        $("#send_log_button").hide()
    },(err)=>{
        console.log("Tracking log ko: "+err)
    })
}

var __err = {
    "sync":"Unexpected error while getting package list!",
    "package":"Unexpected error while getting a package!",
    "packlist":"Unexpected error while getting repository!",
    "tar":"Unexpected error while unpacking a package!",
    "xz":"Unexpected error while uncompressing a package!",
    "install":"Unexpected error while installing a package!",
    "download":"Unexpected error while downloading a package!",
    "fix_runtime": "Unexpected error while organizing system contents!",
}

function handle_error(view,msg,showlog){
    $(".zview").hide()
    if (__err[msg]){
        $(view+" .zmessage").html(__err[msg])
    } else {
        $(view+" .zmessage").html(msg)
    }
    if (showlog) $(view+" .showlog").css("visibility","visible")
    $(view).show()
}

function copyFile(source, target, cb) {
  var cbCalled = false;

  var rd = fs.createReadStream(source);
  rd.on("error", function(err) {
    done(err);
  });
  var wr = fs.createWriteStream(target);
  wr.on("error", function(err) {
    done(err);
  });
  wr.on("close", function(ex) {
    done();
  });
  rd.pipe(wr);

  function done(err) {
    if (!cbCalled) {
      cb(err);
      cbCalled = true;
    }
  }
}

function get_available_versions(){
    return new Promise((resolve,reject)=>{
        $.get(env.remote_versions_list())
            .done((data)=>{
                var sorted_vlist = _.keys(data)
                sorted_vlist = _sort_versions(sorted_vlist)
                console.log(sorted_vlist)
                var result = {sorted:sorted_vlist,raw:data}
                //retrieve the latest installer version
                $.get(env.remote_installer_version())
                    .done((data)=>{
                        clog("Retrieved installer version")
                        clog(data)
                        clog(env.installer_version)
                        try {
                            result.installer = data.version
                        } catch(err){
                            
                        }
                        resolve(result)
                    })
                    .fail((err)=>{
                        clog("Failed to retrieve latest installer version")
                        clog(err)
                        resolve(result)
                    })
            })
            .fail((err)=>{
                console.log(err)
                clog("Error getting versions "+err)
                reject("packlist")
            })
        
    })

}

function get_available_packages(version){
    return new Promise((resolve,reject)=>{
        log("Getting package list...")
        $.get(env.remote_pack_list(version))
            .done((data)=>{
                var repofile = path.join(env.tmp,"repository.json")
                fs.writeFileSync(repofile,JSON.stringify(data))
                console.log(data)
                resolve({file:repofile,data:data})
            }).fail((err)=>{
                clog("Error getting list of packages "+err)
                reject("packlist")
            })
    })
}




function install(offline,ok,not_ok,progress){
    clog("Starting installation at: "+(new Date()).toISOString())
    
    if(offline) {
        //untar
        track_installation("installation_started",env.installer_version)
        untar_offline_py(offline)
            .then((data)=>{
                var prg = 0
                console.log(data)
                var total = data.packs.length
                console.log("untarred")
                start_install_py(data.repofile,data.version,data.patch,data.dir,(sz,tsz,di_st)=>{progress(prg++,total,sz,tsz,di_st)})
                    .then(()=>{
                        track_installation("installation_completed",env.reinstalling)
                        save_installation(offline)
                        if(ok) ok()
                    })
                    .catch((err)=>{ 
                        track_installation("installation_error","installer")
                        clog("error in offline install")
                        clog(err)
                        if(not_ok) not_ok(err)
                    })
            })
            .catch((err)=>{
                track_installation("installation_error","untar")
                clog("error untarring")
                clog(err)
                if(not_ok) not_ok(err)
            })
    } else {
        track_installation("installation_started",env.installer_version)
        $("#version_spinner").show()
        $("#version_selector").hide()
        get_available_versions()
                .then((versions)=>{
                    //versions retrieved, draw select box
                    if(versions.installer && versions.installer!=env.installer_version){
                        //there is a new installer available!
                        obsolete_installer()
                        return
                    }
                    var html=""
                    _.each(versions.sorted,(v,k,l)=>{
                        html+='<option value="'+v+'" '+((k==0)?"selected":"")+'>'+v+'</option>'    
                    })
                    console.log(html)
                    $("#select_install_version").html(html)
                    $("#select_install_version").show()
                    $("#select_install_version").selectpicker("refresh")
                    $("#version_spinner").hide()
                    $("#version_selector").show()
                    $("#start_install_button").click(()=>{
                        // on click retrieve packages
                        var vv = $("#select_install_version option:selected").val()
                        var pp = versions.raw[vv][versions.raw[vv].length-1]
                        get_available_packages(vv)
                            .then((packs)=>{
                                console.log("retrieved packs")
                                console.log(packs)
                                $(".zview").hide()
                                //show installer and run python inst
                                $("#installer_view").show()
                                var prg = 0;
                                var total = packs.data.packs.length*2
                                start_install_py(packs.file,vv,pp,offline,(sz,tsz,di_st)=>{progress(prg++,total,sz,tsz,di_st)})
                                    .then(()=>{
                                        track_installation("installation_completed",env.reinstalling)
                                        save_installation(offline)
                                        if(ok) ok()
                                    })
                                    .catch((err)=>{ track_installation("installation_error","installer")
                                        clog("error in online install")
                                        clog(err)
                                        if(not_ok) not_ok(err)
                                    })
                            })
                            .catch((err)=>{
                                track_installation("installation_error","repo")
                                clog("error retrieving packages")
                                clog(err)
                                if(not_ok) not_ok(err)    
                            })
                    })
                })
                .catch((err)=>{
                    track_installation("installation_error","versions")
                    clog("error retrieving versions")
                    clog(err)
                    if(not_ok) not_ok(err)
                })
    }
}

function untar_offline_py(tarfile){
    var tmpdir = path.join(env.tmp,"offline")
    make_new_dir(tmpdir)
    return new Promise((resolve,reject)=>{
        ZTC.command(["prepare_offline",tarfile,tmpdir],{
            stdout:(line)=>{clog(line)}
        }).then(()=>{
            var packfile;
            var jj;
            var repofile =path.join(tmpdir,"package.json") 
            try {
                jj = fs.readFileSync(repofile,{encoding:"utf8"})
                packfile=JSON.parse(jj)
            } catch(err){
                clog("Error while reading package.json:"+err)
                reject(err)
                return
            }
            packfile.dir = tmpdir
            packfile.repofile = repofile
            resolve(packfile)
        }).catch((err)=>{
            reject(err)
        })
    })
}

function start_install_py(repofile,versions,patch,offline,progress){

    return new Promise((resolve,reject)=>{
    var args = ["install",patch,repofile]
    var sz_re = /^(\[info\]>\s*(Installing|Downloading|Skipping)[^~]*)~\s*([0-9]+)~([0-9]+)$/
    if (offline){
        args.push("--offline")
        args.push(offline)
    }
    if(previous_release) {
        args.push("--community")
        args.push(previous_release)
    }
    ZTC.command(args,{
                        stdout: (line)=>{
                            var res = line.match(sz_re)
                            if (res && progress) {
                                progress(res[3],res[4],res[2].charAt(0));
                                log(res[1])
                            } else log(line) //log everything
                        }
        })
        .then(()=>{
            fix_runtime("python")
                .then(()=>{
                    fix_runtime("browser")
                        .then(()=>{
                            console.log("OK!")
                            resolve() 
                        })
                        .catch((err)=>{
                            reject("fix_runtime")
                        })
                })
                .catch((err)=>{
                    reject("fix_runtime")
                })          
        })
        .catch((err)=>{
            console.log(err)
            reject("install")
        })
    })
}



function make_new_dir(dst){
    try {
        fs.mkdirSync(dst)
    } catch(err){
    }
}


var vrs_re = /^r(0|[1-9][0-9]*)\.(0|[1-9][0-9]*)\.(0|[1-9][0-9]*)$/
function _sort_versions(versions){
    return _.sortBy(versions,(v)=>{
        var res = v.match(vrs_re)
        return -((res[1]<<32)|(res[2]<<16)|res[3])
    })
}

function sformat(s) {
      var fm = [
            Math.floor(s / 60 / 60), // HOURS
            Math.floor(s / 60) % 60, // MINUTES
            s % 60 // SECONDS
      ];
      return $.map(fm, function(v, i) { return ((v < 10) ? '0' : '') + v; }).join(':');
}

function get_all_studio_versions(){
    var srcpath = env.dist
    var dirs = fs.readdirSync(srcpath).filter(function(file) {
        return fs.statSync(path.join(srcpath, file)).isDirectory();
    });
    var versions = []
    _.each(dirs,(v,k,l)=>{
        var res = v.match(vrs_re)
        if (res){
            versions.push(v)
        }
    })
    return _sort_versions(versions)
}

function get_last_studio_version(){
    var versions = get_all_studio_versions()
    return versions[0]
}

function start_studio(version){
    version = version || get_last_studio_version()

    var nw;
    if (env.platform.startsWith("win")) {
        nw = path.join(env.sys,"browser","Zerynth.exe")
    } else if (env.platform.startsWith("linux")) {
        nw = path.join(env.sys,"browser","zerynth")
    } else {
        nw = path.join(env.sys,"browser","Zerynth.app","Contents","MacOS","Zerynth")
    }
    try {
        fs.writeFileSync(path.join(env.cfg,"config.json"),JSON.stringify({version:version}))
    } catch(err){
        console.log("Error while trying to set current version:"+err)
    }
    try {
        fs.writeFileSync(path.join(env.cfg,"root.json"),JSON.stringify({root:process.cwd()}))
    } catch(err){
        console.log("Error while trying to set current root:"+err)
    }
    try {
        fs.writeFileSync(path.join(env.cfg,"root2.json"),JSON.stringify({root:process.cwd()}))
    } catch(err){
        console.log("Error while trying to set current root:"+err)
    }
    var studio = path.join(env.dist,version,"studio")
    clog("Calling:"+nw+" "+studio)
    //window.location.href=studio

    var out = fs.openSync(path.join(env.cfg,'out.log'), 'a');
    var err = fs.openSync(path.join(env.cfg,'out.log'), 'a');
    var child = cp.spawn(nw, [studio], { detached: true, stdio: [ 'ignore', out, err ] });
    child.unref();
    win.close()
}

function save_installation(offline){
    var instfile = path.join(env.cfg,"installation.json")
    if (!fs.existsSync(instfile)){
        fs.writeFileSync(instfile,JSON.stringify({offline:(offline) ? true:false, uid:false}))
    }
    if (env.platform.startsWith("mac")){
        //try adding ztc path to .bashrc
        var bashrc = path.join(env.home,".bash_profile")
        var ztcpath = path.join(env.cwd.replace(/\s/g,"\\ "),"ztc",env.platform);
        try{
            fs.appendFileSync(bashrc,"\n\n########## Zerynth ZTC path\nPATH=$PATH:"+ztcpath+"\nexport PATH\n\n")
        } catch(err){
            clog("Error setting ztc path: "+err)
        }
    }
    //clear temp
    clog("Clearing temporary files...")
    rimraf.sync(env.tmp)
}

var studio_timer;
function setup_studio(){
    var versions
    var current
    var html=""
    versions = get_all_studio_versions()
    console.log("Available versions")
    console.log(versions)
    if (versions.length<=0){
        //Error!
        return null
    }
    try {
        var jj = fs.readFileSync(path.join(env.cfg,"config.json"),{encoding:"utf8"})
        current=JSON.parse(jj).version
    } catch(err){
        console.log("Error while searching for installed version:"+err)
    }
    _.each(versions,(v,k,l)=>{
        html+='<option value="'+v+'" '+((v==current)?"selected":"")+'>'+v+'</option>'
    })
    console.log(html)
    $("#select_version").html(html)
    $('#select_version').selectpicker({
            size: 4,
            width:100
    });
    setTimeout(()=>{
        $("#select_version").show()
        $("#select_version").selectpicker("refresh")
    },100)

    $("#select_version").on("shown.bs.select",()=>{
        if (studio_timer) {
            clearTimeout(studio_timer)
            studio_timer = null
        }
        $("#start_studio_button").css("visibility","visible")
        $("#reinstall_button").css("visibility","visible")
    })
    studio_timer = setTimeout(()=>{
        start_selected_studio()
    },5000)
    return versions
}

function reinstall(){
    if (studio_timer) clearTimeout(studio_timer)
    $(".zview").hide()
    $("#installer_welcome_view").show()    
}

function start_selected_studio(){
    var selected = $('#select_version option:selected').val();
    clog(selected)
    start_studio(selected)
}


function install2(){
    $(".zview").hide()
    $("#installer_choose_view").show()
}

function has_patches(){

    var patchfile = path.join(env.tmp,"patchfile.json")
    
    
    if (!fs.existsSync(patchfile)) {
        clog("no patches, skipping...")
        return false
    }
    var jspatch = fs.readFileSync(patchfile,{encoding:"utf-8"})
    var jsp
    try {
        jsp = JSON.parse(jspatch)
    } catch(err) {
        clog("bad json in patches...")
        return false
    }

    return jsp
}

function install_patches(jsp,pcb){
    track_installation("update_started",jsp.version)
    async.eachSeries(jsp.packs,(pack,cb)=>{
        var destdir = pack.destdir
        var srcdir  = pack.srcdir
        log("Patching "+pack.destdir)
        if (srcdir=="" || (fs.existsSync(srcdir))) {
            //avoid deleting destdir if no srcdir present
            rimraf(destdir,()=>{
                if (srcdir!="") {
                    make_new_dir(destdir)
                    fs.copy(srcdir,destdir,(err)=>{
                        if (err) {
                            cb(err)
                            return
                        }
                        cb()
                    })
                } else {
                    //package deleted
                    cb()
                }
            })
        } else cb("Missing source dir!")
    },(eee)=>{
        if(!eee) {
            //save cfg/patches.json
            track_installation("update_completed","ok")
            fs.writeFileSync(path.join(env.dist,jsp.version,"patches.json"),JSON.stringify(jsp.patch))
            //delete patchfile and patchdir
        } else {
            track_installation("update_error","ko")
        }
        fs.removeSync(path.join(env.tmp,"patchfile.json"))
        fs.removeSync(path.join(env.tmp,"patch"))
        if (pcb) pcb(eee)
    })
}

function obsolete_installer(){
    $(".zview").hide()
    $("#installer_obsolete_view").show()
}

